import Table from "@mui/material/Table";
import TableBody from "@mui/material/TableBody";
import TableCell from "@mui/material/TableCell";
import TableContainer from "@mui/material/TableContainer";
import TableHead from "@mui/material/TableHead";
import TableRow from "@mui/material/TableRow";
import Paper from "@mui/material/Paper";
import { Check, RotateCcw, X } from "lucide-react";
import {
  authInstance,
  devicesStore,
  Modal,
  snapshotStore,
  vehicleStore,
} from "@shared";
import { useEffect, useState } from "react";
import { observer } from "mobx-react-lite";
import { Button, Checkbox, Typography } from "@mui/material"; // Import Typography for the modal message

const formatDate = (dateString: string | undefined) => {
  if (!dateString) return "Нет данных";

  try {
    const date = new Date(dateString);
    return new Intl.DateTimeFormat("ru-RU", {
      day: "2-digit",
      month: "2-digit",
      year: "numeric",
      hour: "2-digit",
      minute: "2-digit",
      second: "2-digit",
      hour12: false,
    }).format(date);
  } catch (error) {
    console.error("Error formatting date:", error);
    return "Некорректная дата";
  }
};

function createData(
  uuid: string,
  online: boolean,
  lastUpdate: string,
  gps: boolean,
  media: boolean,
  connection: boolean
) {
  return { uuid, online, lastUpdate, gps, media, connection };
}

// Keep the rows function as you provided it, without additional filters
const rows = (vehicles: any[]) => {
  return vehicles.map((vehicle) => {
    return createData(
      vehicle?.vehicle?.tail_number ?? "1243000", // Using tail_number as UUID, as in your original code
      vehicle?.device_status?.online ?? false,
      vehicle?.device_status?.last_update,
      vehicle?.device_status?.gps_ok,
      vehicle?.device_status?.media_service_ok,
      vehicle?.device_status?.is_connected
    );
  });
};

export const DevicesTable = observer(() => {
  const {
    devices,
    getDevices,
    // uuid, // This 'uuid' from devicesStore refers to a *single* selected device, not for batch actions.
    setSelectedDevice, // Useful for individual device actions like 'Reload Status'
    sendSnapshotModalOpen,
    toggleSendSnapshotModal,
  } = devicesStore;
  const { snapshots, getSnapshots } = snapshotStore;
  const { getVehicles } = vehicleStore;
  const [selectedDevices, setSelectedDevices] = useState<string[]>([]);

  // Get the current list of rows displayed in the table
  const currentRows = rows(devices);

  useEffect(() => {
    const fetchData = async () => {
      await getVehicles();
      await getDevices();
      await getSnapshots();
    };
    fetchData();
  }, []);

  // Determine if all visible devices are selected
  const isAllSelected =
    currentRows.length > 0 && selectedDevices.length === currentRows.length;

  const handleSelectAllDevices = () => {
    if (isAllSelected) {
      // If all are currently selected, deselect all
      setSelectedDevices([]);
    } else {
      // Otherwise, select all device UUIDs from the current rows
      setSelectedDevices(currentRows.map((row) => row.uuid));
    }
  };

  const handleSelectDevice = (event: React.ChangeEvent<HTMLInputElement>) => {
    const deviceUuid = event.target.value;
    if (event.target.checked) {
      setSelectedDevices((prevSelected) => [...prevSelected, deviceUuid]);
    } else {
      setSelectedDevices((prevSelected) =>
        prevSelected.filter((uuid) => uuid !== deviceUuid)
      );
    }
  };

  // This function now opens the modal for selected devices
  const handleOpenSendSnapshotModal = () => {
    if (selectedDevices.length > 0) {
      toggleSendSnapshotModal();
    }
  };

  const handleReloadStatus = async (uuid: string) => {
    setSelectedDevice(uuid); // Set the active device in store for context if needed
    await authInstance.post(`/devices/${uuid}/request-status`);
    await getDevices(); // Refresh devices after status request
  };

  // This function now handles sending snapshots to ALL selected devices
  const handleSendSnapshotAction = async (snapshotId: string) => {
    try {
      for (const deviceUuid of selectedDevices) {
        console.log(`Sending snapshot ${snapshotId} to device ${deviceUuid}`);
        // Ensure you are using the correct API endpoint for force-snapshot
        await authInstance.post(`/devices/${deviceUuid}/force-snapshot`, {
          snapshot_id: snapshotId,
        });
      }
      // After all requests are sent
      await getDevices(); // Refresh the device list to show updated status
      setSelectedDevices([]); // Clear the selection
      toggleSendSnapshotModal(); // Close the modal
    } catch (error) {
      console.error("Error sending snapshots:", error);
      // You might want to show an error notification to the user here
    }
  };

  return (
    <>
      <TableContainer component={Paper}>
        <div className="flex justify-end p-3 gap-3">
          {" "}
          {/* Changed gap to 3 for slightly less space */}
          <Button
            variant="contained"
            color="primary"
            onClick={handleSelectAllDevices}
          >
            {isAllSelected ? "Снять выбор со всех" : "Выбрать все"}
          </Button>
          <Button
            variant="contained"
            color="primary"
            disabled={selectedDevices.length === 0}
            onClick={handleOpenSendSnapshotModal} // Call the new handler
          >
            Отправить снапшот ({selectedDevices.length})
          </Button>
        </div>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead>
            <TableRow>
              <TableCell align="center" padding="checkbox">
                {" "}
                {/* Added padding="checkbox" */}
                <Checkbox
                  checked={isAllSelected}
                  onChange={handleSelectAllDevices}
                  inputProps={{ "aria-label": "select all devices" }}
                />
              </TableCell>
              <TableCell align="center">Бортовой номер</TableCell>
              <TableCell align="center">Онлайн</TableCell>
              <TableCell align="center">Последнее обновление</TableCell>
              <TableCell align="center">ГПС</TableCell>
              <TableCell align="center">Медиа-данные</TableCell>
              <TableCell align="center">Подключение</TableCell>
              <TableCell align="center">Перезапросить</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {currentRows.map((row) => (
              <TableRow
                key={row.uuid} // Use row.uuid as key for consistent rendering
                sx={{ "&:last-child td, &:last-child th": { border: 0 } }}
              >
                <TableCell align="center" padding="checkbox">
                  {" "}
                  {/* Added padding="checkbox" */}
                  <Checkbox
                    className="h-full"
                    onChange={handleSelectDevice}
                    value={row.uuid}
                    checked={selectedDevices.includes(row.uuid)} // THIS IS THE KEY CHANGE
                  />
                </TableCell>

                <TableCell align="center" component="th" scope="row">
                  {row.uuid}
                </TableCell>
                <TableCell align="center">
                  {row.online ? (
                    <Check className="m-auto text-green-500" />
                  ) : (
                    <X className="m-auto text-red-500" />
                  )}
                </TableCell>
                <TableCell align="center">
                  {formatDate(row.lastUpdate)}
                </TableCell>
                <TableCell align="center">
                  {row.gps ? (
                    <Check className="m-auto text-green-500" />
                  ) : (
                    <X className="m-auto text-red-500" />
                  )}
                </TableCell>
                <TableCell align="center">
                  {row.media ? (
                    <Check className="m-auto text-green-500" />
                  ) : (
                    <X className="m-auto text-red-500" />
                  )}
                </TableCell>
                <TableCell align="center">
                  {row.connection ? (
                    <Check className="m-auto text-green-500" />
                  ) : (
                    <X className="m-auto text-red-500" />
                  )}
                </TableCell>
                <TableCell align="center">
                  <button onClick={() => handleReloadStatus(row.uuid)}>
                    <RotateCcw className="m-auto" />
                  </button>
                </TableCell>
              </TableRow>
            ))}
          </TableBody>
        </Table>
      </TableContainer>
      <Modal open={sendSnapshotModalOpen} onClose={toggleSendSnapshotModal}>
        <Typography variant="h6" component="p" sx={{ mb: 2 }}>
          Выбрать снапшот для{" "}
          <strong className="text-blue-600">{selectedDevices.length}</strong>{" "}
          устройств
        </Typography>
        <div className="mt-5 flex flex-col gap-2 max-h-[300px] overflow-y-auto">
          {snapshots && snapshots.length > 0 ? (
            snapshots.map((snapshot) => (
              <Button
                variant="outlined"
                onClick={() => handleSendSnapshotAction(snapshot.ID)}
                sx={{
                  p: 1.5, // Adjust padding
                  borderRadius: 2, // Adjust border radius
                  backgroundColor: "white", // Ensure background is white
                  "&:hover": {
                    backgroundColor: "grey.100", // Light hover effect
                  },
                }}
                key={snapshot.ID}
              >
                {snapshot.Name}
              </Button>
            ))
          ) : (
            <Typography variant="body2" color="textSecondary">
              Нет доступных снапшотов.
            </Typography>
          )}
        </div>
      </Modal>
    </>
  );
});
